import {
  Singleton, Metadata, Interceptor, I18n, Cookie, Router, Http, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  DialogService, Select, Menu, Modal, ModalService,
} from '@zeedhi/common';
import store from '../store';
import router from '../router';

@Singleton
export class AppController {
  public isHomeActive?: boolean;

  public get nmOperador() {
    return localStorage.getItem('NMOPERADOR') || '';
  }
  
  public confirmLogoutModal!: Modal;

  public metaLanguages: IDictionary = {
    pt_br: 'pt-BR',
    es_cl: 'es-CL',
    es_es: 'es-ES',
    es_co: 'es-CO',
    en_us: 'en-US',
  };
  
  public metaCurrency: IDictionary = {
    pt_br: 'BRL',
    es_cl: 'CLP',
    es_es: 'EUR',
    es_co: 'COP',
    en_us: 'USD',
  };
  
  public get nmCentral() {
    const central = Metadata.getInstance<Select>('CDFILCENTPME');
    return central.value ? (central.value as IDictionary).NMFILIAL : '';
  }
  
  public get cdCentral() {
    return store.getters.cdCentral;
  }

  public showSessionDialog() {
    DialogService.show(
      {
        name: 'logout',
        type: 'warning',
        title: 'pme::sessaoExpirou',
        text: 'pme::redirecionadoLogin',
        buttons: [
          {
            name: 'dialogButton', label: 'pme::OK', component: 'ZdButton', events: { click: () => this.redirectLogin() },
          },
        ],
      },
    );
    setTimeout(() => {
      DialogService.hide();
      Router.replace('/login');
    }, 3600000);
  }
  
  public redirectLogin() {
    DialogService.hide();
    Router.replace('/login');
  }

  get isMini() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    return menu.mini;
  }
  
  get isNotMini() {
    return !this.isMini;
  }
  
  public routerAcessMini() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    menu.showSearch = false;
    menu.mini = !menu.mini;
    menu.clearSearch();
  }
  
  public onInit() {
    this.setLangDefaultCookie();
    
    Interceptor.addRequestSuccess((axiosConfig) => {
      const token = Cookie.getCookie('token');
      const cdoperador = localStorage.getItem('CDOPERADOR');
      if (!token || token === 'undefined' || !cdoperador || cdoperador === 'undefined') {
        Cookie.deleteCookie('token');
        Router.replace('/login');
      } else {
        axiosConfig.headers.Authorization = `Bearer ${token}`;
        axiosConfig.headers.oAuthToken = token;

        if (axiosConfig.params && axiosConfig.params.search) {
          axiosConfig.params.search = axiosConfig.params.search.toUpperCase();
        }
      }
      axiosConfig.headers.post['Content-Type'] = 'application/json;charset=utf-8';
      axiosConfig.headers.post['Access-Control-Allow-Origin'] = '*';
      axiosConfig.headers.OAuthKeepConnected = 'No';

      // if (axiosConfig.url === '/authenticate') store.commit('loading', true);
      return axiosConfig;
    });

    Interceptor.addResponseSuccess((axiosConfig) => {
      if (axiosConfig.data.dataset) {
        axiosConfig.data.data = axiosConfig.data.dataset.data;
      }

      // if (axiosConfig.config.url === '/authenticate') store.commit('loading', false);
      return axiosConfig;
    });

    Interceptor.addResponseFail((axiosError) => {
      let errorMessage = '';
      const { error, errorCode } = axiosError.response?.data;
      if (errorCode === 401) {
        Cookie.deleteCookie('token');
        Router.replace('/login');
      } else if (errorCode === 403) {
        this.showSessionDialog();
      } else if (error) {
        // temporarily
        errorMessage = error.replace('Exception: ', '');
      }
      
      // if (axiosError.config.url === '/authenticate') store.commit('loading', false);
      
      return Promise.reject(errorMessage);
    });
  }
  
  private setLangDefaultCookie() {
    const language = Cookie.getCookie('language');
    if (!language || language === 'undefined') {
      Cookie.setCookie('language', 'pt_br');
    } else {
      I18n.changeLanguage(this.metaLanguages[language]);
    }
  }
  
  public setUniDefault({ component }: IEventParam<Select>) {
    const currentRoute = Router.getCurrentRoute() as IDictionary;
    if (currentRoute.name !== 'login') {
      const actualCdCentral = localStorage.getItem('CDFILIAL');
      const msgParamCentral = () => {
        DialogService.show(
          {
            name: 'paramCentral',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::parametrizacaoCentral',
            buttons: [
              {
                name: 'dialogButton', label: 'pme::OK', component: 'ZdButton', events: { click: () => DialogService.hide() },
              },
            ],
          },
        );
      };
      
      if (actualCdCentral && actualCdCentral !== 'null') {
        Http.get('/central').then((response) => {
          Http.get('/unidadeCentral').then((resultUnid) => {
            if (response.data.data.length !== 0) {
              const defaultCentral = response.data.data.filter((element: IDictionary) => element.CDFILIAL === actualCdCentral);
              const cdfilcentpme = resultUnid.data.data.filter((element: IDictionary) => element.CDAGREGADA === actualCdCentral);
              if (defaultCentral.length !== 0) component.setValue(defaultCentral[0]);
              else if (cdfilcentpme.length !== 0) component.setValue(cdfilcentpme[0]);
              else component.setValue(response.data.data[0]);
            } else msgParamCentral();
          });
        });
      } else {
        Http.post('/defaultCentral').then((response) => {
          if (response.data.dataset.central.length !== 0) {
            component.datasource.get();
            component.setValue(response.data.dataset.central[0]);
            store.commit('cdCentral', response.data.dataset.central[0].CDFILIAL);
          } else msgParamCentral();
        });
      }
    }
  }

  public setInfoCentral({ component }: IEventParam<Select>) {
    const cdCentral = component.value ? component.value.CDFILIAL : null;
    localStorage.setItem('CDFILIAL', cdCentral);
    store.commit('cdCentral', cdCentral);
  }

  public getCdCentral() {
    return this.cdCentral;
  }

  public getNmCentral() {
    return this.nmCentral;
  }
  
  public async createModal() {
    this.confirmLogoutModal = await ModalService.createFromJson('/modals/confirmLogoutModal', true);
  }
  
  public async openConfirmLogoutModal() {
    await this.createModal();
    this.confirmLogoutModal.show();
  }
  
  public hideConfirmLogoutModal() {
    this.confirmLogoutModal.destroy();
  }
  
  public logout() {
    this.confirmLogoutModal.destroy();
    Http.post('/logout', {}).then(() => {
      Cookie.deleteCookie('token');
      localStorage.clear();
      store.commit('cdCentral', null);
      router.push('/login');
    });
  }

  public toggleMenu() {
    const menu = Metadata.getInstance<Menu>('main-menu');
    const isMobile = window.innerWidth <= menu.mobileBreakpoint;
    if (isMobile) {
      menu.drawer = !menu.drawer;
      menu.mini = false;
      menu.showSearch = true;
    } else {
      menu.drawer = true;
      menu.mini = !menu.mini;
    
      if (!this.isMini) {
        menu.showSearch = true;
        setTimeout(() => {
          const searchInput = document.querySelector('.zd-menu-search-input input');
          if (searchInput) (searchInput as HTMLElement).focus();
        }, 100);
      } else {
        menu.showSearch = false;
        const firstMenuItem = document.querySelector('.zd-menu-link, .zd-menu-group > div');
        if (firstMenuItem) (firstMenuItem as HTMLElement).focus();
      }
    }
  }
}
